@@ -20,7 +20,7 @@ class _WiserTemperatureFunctions(object):
     # Support Functions
     # -----------------------------------------------------------
     @staticmethod
-    def _to_wiser_temp(temp: float, type: str = "heating", units:WiserUnitsEnum = WiserUnitsEnum.metric) -> int:
+    def _to_wiser_temp(temp: float, type: str = "set_heating", units:WiserUnitsEnum = WiserUnitsEnum.metric) -> int:
         """
         Converts from degrees C to wiser hub format
         param temp: The temperature to convert
@@ -424,39 +424,54 @@ class _WiserHubCapabilitiesInfo:
         self._data = data
 
     @property
-    def all(self) -> str:
+    def all(self) -> dict:
         "Get the list of capabilities"
-        return self._data
+        return dict(self._data)
+
+    @property
+    def smartplug(self):
+        return self._data.get("SmartPlug", False)
+
+    @property
+    def itrv(self):
+        return self._data.get("ITRV", False)
+
+    @property
+    def roomstat(self):
+        return self._data.get("Roomstat", False)
+
+    @property
+    def ufh(self):
+        return self._data.get("UFH", False)
+
+    @property
+    def ufh_floor_temp_sensor(self):
+        return self._data.get("UFHFloorTempSensor", False)
+
+    @property
+    def ufh_dew_sensor(self):
+        return self._data.get("UFHDewSensor", False)
+
+    @property
+    def hact(self):
+        return self._data.get("HACT", False)
+
+    @property
+    def lact(self):
+        return self._data.get("LACT", False)
+
+    @property
+    def light(self):
+        return self._data.get("Light", False)
+
+    @property
+    def shutter(self):
+        return self._data.get("Shutter", False)
+
+    @property
+    def load_controller(self):
+        return self._data.get("LoadController", False)
     
-    def get_capability(self, name) -> bool:
-        """
-        Gets a capability value from the hub capabilities
-        param name: name of capability
-        return: bool
-        """
-        try:
-            return [capability.value for capability in self.capabilities if capability.key == name][0]
-        except IndexError:
-            return None
-
-class _WiserOutputRange(object):
-    """ Data structure for min/max output range"""
-    def __init__(self, data: dict):
-        self._data = data
-
-    @property
-    def min(self) -> int:
-        """Get min value"""
-        if self._data:
-            return self._data.get("Minimum")
-        return None
-
-    @property
-    def max(self) -> int:
-        """Get max value"""
-        if self._data:
-            return self._data.get("Maximum")
-        return None
 
 
     
